@@ -9,6 +9,7 @@ import pprint
 import re
 import shutil
 import subprocess
+import sys
 
 import markdoc
 from markdoc.builder import Builder
@@ -25,7 +26,7 @@ def command(function):
     @wraps(function)
     def wrapper(config, args):
         if not args.quiet:
-            print '--> markdoc', cmd_name
+            print >>sys.stderr, '--> markdoc', cmd_name
         return function(config, args)
     wrapper.parser = parser
     
@@ -53,12 +54,12 @@ def init(_, args):
     else:
         destination = p.abspath(args.destination)
     
-    if os.path.exists(destination) and os.listdir(destination):
+    if p.exists(destination) and os.listdir(destination):
         init.parser.error("destination isn't empty")
-    elif not os.path.exists(destination):
+    elif not p.exists(destination):
         log.info('makedirs %s' % destination)
         os.makedirs(destination)
-    elif not os.path.isdir(destination):
+    elif not p.isdir(destination):
         init.parser.error("destination isn't a directory")
     
     log.info('mkdir %s/.templates/' % destination)
@@ -76,10 +77,61 @@ def init(_, args):
     finally:
         fp.close()
     
+    if args.vcs_ignore:
+        config = markdoc.config.Config.for_directory(destination)
+        args = vcs_ignore.parser.parse_args([args.vcs_ignore])
+        vcs_ignore(config, args)
+    
     log.info('Wiki initialization complete')
     log.info('Your new wiki is at: %s' % destination)
 
-init.parser.add_argument('destination')
+init.parser.add_argument('destination', default=None,
+    help="Create wiki here (if omitted, defaults to current directory)")
+init.parser.add_argument('--vcs-ignore', choices=['hg', 'git', 'cvs', 'bzr'],
+    help="Create an ignore file for the specified VCS.")
+
+
+@command
+def vcs_ignore(config, args):
+    """Create a VCS ignore file for a wiki."""
+    
+    log = logging.getLogger('markdoc.vcs-ignore')
+    log.info('Creating ignore file for %s' % args.vcs)
+    wiki_root = config['meta.root'] # shorter local alias.
+    
+    ignore_file_lines = []
+    ignore_file_lines.append(p.relpath(config.html_dir, start=wiki_root))
+    ignore_file_lines.append(p.relpath(config.temp_dir, start=wiki_root))
+    if args.vcs == 'hg':
+        ignore_file_lines.insert(0, 'syntax: glob')
+        ignore_file_lines.insert(1, '')
+    
+    if args.output == '-':
+        log.info('Writing ignore file to stdout')
+        fp = sys.stdout
+    else:
+        if not args.output:
+            filename = p.join(wiki_root, '.%signore' % args.vcs)
+        else:
+            filename = p.join(wiki_root, args.output)
+        log.info('Writing ignore file to %s' % p.relpath(filename, start=wiki_root))
+        fp = open(filename, 'w')
+    
+    try:
+        fp.write('\n'.join(ignore_file_lines) + '\n')
+    finally:
+        if fp is not sys.stdout:
+            fp.close()
+    
+    log.info('Ignore file written.')
+
+vcs_ignore.parser.add_argument('vcs', default='hg', nargs='?',
+    choices=['hg', 'git', 'cvs', 'bzr'],
+    help="Create ignore file for specified VCS (default 'hg')")
+vcs_ignore.parser.add_argument('-o', '--output', default=None, metavar='FILENAME',
+    help="Write output to the specified filename, relative to the wiki root. "
+         "Default is to generate the filename from the VCS. "
+         "'-' will write to stdout.")
 
 
 ## Cleanup
@@ -127,7 +179,7 @@ def sync_static(config, args):
     command = 'rsync -vaxq --ignore-errors --exclude=".*" --exclude="_*"'.split()
     display_cmd = command[:]
     
-    if config.setdefault('use-default-static', True):
+    if config['use-default-static']:
         # rsync needs the paths to have trailing slashes to work correctly.
         command.append(p.join(markdoc.default_static_dir, ''))
         display_cmd.append(p.basename(markdoc.default_static_dir) + '/')
@@ -163,7 +215,7 @@ def sync_html(config, args):
     command.append(p.join(config.temp_dir, ''))
     display_cmd.append(p.basename(config.temp_dir) + '/')
     
-    if config.setdefault('use-default-static', True):
+    if config['use-default-static']:
         command.append(p.join(markdoc.default_static_dir, ''))
         display_cmd.append(p.basename(markdoc.default_static_dir) + '/')
     
@@ -218,7 +270,7 @@ def build_listing(config, args):
     
     log = logging.getLogger('markdoc.build-listing')
     
-    list_basename = config.setdefault('listing-filename', '_list.html')
+    list_basename = config['listing-filename']
     builder = Builder(config)
     generate_listing = config.get('generate-listing', 'always').lower()
     always_list = True
@@ -253,6 +305,7 @@ def build_listing(config, args):
             log.info("cp %s/%s %s/%s" % (directory, list_basename, directory, 'index.html'))
             shutil.copyfile(list_filename, p.join(fs_dir, 'index.html'))
 
+
 ## Serving
 
 IPV4_RE = re.compile(r'^(25[0-5]|2[0-4]\d|[0-1]?\d?\d)(\.(25[0-5]|2[0-4]\d|[0-1]?\d?\d)){3}$')
@@ -268,8 +321,12 @@ def serve(config, args):
     app = MarkdocWSGIApplication(config)
     
     extra_config = {}
-    if args.port:
-        extra_config['port'] = args.port
+    extra_config['port'] = args.port
+    extra_config['numthreads'] = args.num_threads
+    if args.server_name:
+        extra_config['server_name'] = args.server_name
+    extra_config['request_queue_size'] = args.queue_size
+    extra_config['timeout'] = args.timeout
     if args.interface:
         if not IPV4_RE.match(args.interface):
             serve.parser.error('invalid interface specifier: %r' % args.interface)
@@ -286,7 +343,15 @@ def serve(config, args):
         log.info('Shutting down gracefully')
         server.stop()
 
-serve.parser.add_argument('-p', '--port', type=int, default=None,
+serve.parser.add_argument('-p', '--port', type=int, default=8008,
     help="Listen on specified port (default is 8008)")
 serve.parser.add_argument('-i', '--interface', default=None,
     help="Bind to specified interface (defaults to loopback only)")
+serve.parser.add_argument('-t', '--num-threads', type=int, default=10, metavar='N',
+    help="Use N threads to handle requests (default is 10)")
+serve.parser.add_argument('-n', '--server-name', default=None, metavar='NAME',
+    help="Use an explicit server name (default to an autodetected value)")
+serve.parser.add_argument('-q', '--queue-size', type=int, default=5, metavar='SIZE',
+    help="Set request queue size (default is 5)")
+serve.parser.add_argument('--timeout', type=int, default=10,
+    help="Set the socket timeout for connections (default is 10)")
